@@ -12,19 +12,19 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
+ *
+ * This file was modified by Deephaven Data Labs.
+ *
  */
 
 package org.jpy;
 
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.util.ArrayList;
-import java.util.Map;
+import java.util.function.Supplier;
 
-import static org.jpy.PyLibConfig.JPY_LIB_KEY;
-import static org.jpy.PyLibConfig.OS;
-import static org.jpy.PyLibConfig.PYTHON_LIB_KEY;
-import static org.jpy.PyLibConfig.getOS;
-import static org.jpy.PyLibConfig.getProperty;
+import static org.jpy.PyLibConfig.*;
 
 /**
  * Represents the library that provides the Python interpreter (CPython).
@@ -161,6 +161,13 @@ public class PyLib {
      */
     public static native boolean isPythonRunning();
 
+    /**
+     * Delegates to {@link #startPython(int, String...)} with {@code flags = Diag.F_OFF}.
+     */
+    public static void startPython(String... extraPaths) {
+        startPython(Diag.F_OFF, extraPaths);
+    }
+
     /**
      * Starts the Python interpreter. It does the following:
      * <ol>
@@ -169,11 +176,11 @@ public class PyLib {
      * <li>Imports the 'jpy' extension module, if not already done.</li>
      * </ol>
      *
+     * @param flags      If non-zero, is passed to {@link Diag#setFlags(int)} before python is started
      * @param extraPaths List of paths that will be prepended to Python's 'sys.path'.
      * @throws RuntimeException if Python could not be started or if the 'jpy' extension module could not be loaded.
      */
-    public static void startPython(String... extraPaths) {
-
+    public static void startPython(int flags, String... extraPaths) {
         ArrayList<File> dirList = new ArrayList<>(1 + extraPaths.length);
 
         File moduleDir = new File(dllFilePath).getParentFile();
@@ -198,7 +205,9 @@ public class PyLib {
             for (String path : extraPaths) {
                 System.out.printf("org.jpy.PyLib:   %s%n", path);
             }
-            Diag.setFlags(Diag.F_EXEC);
+            Diag.setFlags(Diag.F_EXEC | flags);
+        } else if (flags != 0) {
+            Diag.setFlags(flags);
         }
 
         startPython0(extraPaths);
@@ -206,6 +215,25 @@ public class PyLib {
 
     static native boolean startPython0(String... paths);
 
+    /**
+     * Does the equivalent of setting the PYTHONHOME environment variable.  If used,
+     * this must be called prior to calling {@code startPython()}.
+     * Supported for Python 2.7, and Python 3.5 or higher
+     * @param pythonHome Path to Python Home (must be less than 256 characters!)
+     * @return true if successful, false if it fails
+     */
+    public static native boolean setPythonHome(String pythonHome);
+
+    /**
+     * Useful for virtual environments, helps in setting sys.prefix/exec_prefix.
+     * If used, this must be called prior to calling {@code startPython()}.
+     * @param programName Path to Python executable (must be less than 256 characters!)
+     * @return true if successful, false if it fails
+     * @see <a href="https://docs.python.org/2/c-api/init.html#c.Py_SetProgramName">Py_SetProgramName (2)</a>
+     * @see <a href="https://docs.python.org/3/c-api/init.html#c.Py_SetProgramName">Py_SetProgramName (3)</a>
+     */
+    public static native boolean setProgramName(String programName);
+
     /**
      * @return The Python interpreter version string.
      */
@@ -222,6 +250,7 @@ public class PyLib {
      * For more information refer to https://github.com/bcdev/jpy/issues/70
      */
     public static void stopPython() {
+        PyObject.cleanup();
         if (!STOP_IS_NO_OP) {
             stopPython0();
         }
@@ -232,26 +261,124 @@ public class PyLib {
     @Deprecated
     public static native int execScript(String script);
 
-    static native long executeCode(String code, int start, Map<String, Object> globals, Map<String, Object> locals);
+    /**
+     * Callers must close the returned reference with {@link #decRef(long)}.
+     */
+    static native long executeCode(String code, int start, Object globals, Object locals);
+
+    /**
+     * Callers must close the returned reference with {@link #decRef(long)}.
+     */
+    static native long executeScript
+            (String file, int start, Object globals, Object locals) throws FileNotFoundException;
+
+    public static native PyObject getMainGlobals();
+
+    /**
+     * Return a dictionary of the global variables in the current execution frame, or NULL if no
+     * frame is currently executing.
+     *
+     * @return the current globals, or null
+     * @see <a href="https://docs.python.org/2/c-api/reflection.html#c.PyEval_GetGlobals">PyEval_GetGlobals (2)</a>
+     * @see <a href="https://docs.python.org/3/c-api/reflection.html#c.PyEval_GetGlobals">PyEval_GetGlobals (3)</a>
+     */
+    public static native PyObject getCurrentGlobals();
+
+    /**
+     * Return a dictionary of the local variables in the current execution frame, or NULL if no
+     * frame is currently executing.
+     *
+     * @return the current locals, or null
+     * @see <a href="https://docs.python.org/2/c-api/reflection.html#c.PyEval_GetLocals">PyEval_GetLocals (2)</a>
+     * @see <a href="https://docs.python.org/3/c-api/reflection.html#c.PyEval_GetLocals">PyEval_GetLocals (3)</a>
+     */
+    public static native PyObject getCurrentLocals();
+
+    static native PyObject copyDict(long pyPointer);
 
     static native void incRef(long pointer);
 
     static native void decRef(long pointer);
 
+    static native void decRefs(long[] pointers, int len);
+
     static native int getIntValue(long pointer);
 
+    static native long getLongValue(long pointer);
+
+    static native boolean getBooleanValue(long pointer);
+
     static native double getDoubleValue(long pointer);
 
     static native String getStringValue(long pointer);
 
     static native Object getObjectValue(long pointer);
 
+    static native boolean isConvertible(long pointer);
+    static native boolean pyNoneCheck(long pointer);
+    static native boolean pyDictCheck(long pointer);
+    static native boolean pyListCheck(long pointer);
+    static native boolean pyBoolCheck(long pointer);
+    static native boolean pyIntCheck(long pointer);
+    static native boolean pyLongCheck(long pointer);
+    static native boolean pyFloatCheck(long pointer);
+    static native boolean pyStringCheck(long pointer);
+    static native boolean pyCallableCheck(long pointer);
+    static native boolean pyFunctionCheck(long pointer);
+    static native boolean pyModuleCheck(long pointer);
+    static native boolean pyTupleCheck(long pointer);
+
+    static native long getType(long pointer);
+
+    static native String str(long pointer);
+
+    static native String repr(long pointer);
+
+    static native long hash(long pointer);
+
+    static native boolean eq(long pointer1, Object other);
+
+    static native PyObject newDict();
+
+    /**
+     * https://docs.python.org/2/c-api/dict.html#c.PyDict_Keys
+     * https://docs.python.org/3/c-api/dict.html#c.PyDict_Keys
+     * @return Return a PyListObject containing all the keys from the dictionary.
+     */
+    static native PyObject pyDictKeys(long pointer);
+
+    /**
+     * https://docs.python.org/2/c-api/dict.html#c.PyDict_Values
+     * https://docs.python.org/3/c-api/dict.html#c.PyDict_Values
+     * @return Return a PyListObject containing all the values from the dictionary p.
+     */
+    static native PyObject pyDictValues(long pointer);
+
+    /**
+     * Determine if dictionary dict contains key.
+     * This is equivalent to the Python expression `key in dict`
+     *
+     * https://docs.python.org/2/c-api/dict.html#c.PyDict_Contains
+     * https://docs.python.org/3/c-api/dict.html#c.PyDict_Contains
+     *
+     * @param dict     the dictionary
+     * @param key      the key
+     * @param keyClass Optional type for converting the key to a Python object
+     * @return True iff key is in dict.
+     */
+    static native <T> boolean pyDictContains(long dict, T key, Class<? extends T> keyClass);
+
     static native <T> T[] getObjectArrayValue(long pointer, Class<? extends T> itemType);
 
+    /**
+     * Callers must close the returned reference with {@link #decRef(long)}.
+     */
     static native long importModule(String name);
 
     /**
      * Gets the value of a given Python attribute as Python object pointer.
+     * <p>
+     * Callers must close the returned reference with {@link #decRef(long)}.
      *
      * @param pointer Identifies the Python object which contains the attribute {@code name}.
      * @param name    The attribute name.
@@ -284,6 +411,29 @@ public class PyLib {
      */
     static native <T> void setAttributeValue(long pointer, String name, T value, Class<? extends T> valueType);
 
+    /**
+     * Deletes the Python attribute given by {@code name} of the Python object pointed to by {@code pointer}.
+     * <p>
+     *
+     * @param pointer Identifies the Python object which contains the attribute {@code name}.
+     * @param name    The attribute name.
+     */
+    static native void delAttribute(long pointer, String name);
+
+    /**
+     * Checks for the existence the Python attribute given by {@code name} of the Python object pointed to by {@code pointer}.
+     * <p>
+     *
+     * @param pointer Identifies the Python object which contains the attribute {@code name}.
+     * @param name    The attribute name.
+     * @return true if the Python object has an attribute named {@code name}
+     */
+    static native boolean hasAttribute(long pointer, String name);
+
+    public static native boolean hasGil();
+
+    public static native <T> T ensureGil(Supplier<T> runnable);
+
     /**
      * Calls a Python callable and returns the resulting Python object.
      * <p>
@@ -291,6 +441,8 @@ public class PyLib {
      * Python objects.
      * The {@code args} array may also contain objects of type {@code PyObject}.
      * These will be directly translated into the corresponding Python objects without conversion.
+     * <p>
+     * Callers must close the returned reference with {@link #decRef(long)}.
      *
      * @param pointer    Identifies the Python object which contains the callable {@code name}.
      * @param methodCall true, if this is a call of a method of the Python object pointed to by {@code pointer}.
@@ -400,7 +552,17 @@ public class PyLib {
     private PyLib() {
     }
 
+    /**
+     * We call this method to ensure that this class gets loaded, and thus the static block gets run
+     *
+     * We don't technically need this method. Callers could instead rely on reflection, using
+     * {@link Class#forName(String)}, but this method is better because it is much more explicit.
+     */
+    static void dummyMethodForInitialization() { }
+
     static {
+        // see documentation in PyLibInitializer for explanation
+        PyLibInitializer.pyLibInitialized = true;
         if (DEBUG) System.out.println("org.jpy.PyLib: entered static initializer");
         loadLib();
         if (DEBUG) System.out.println("org.jpy.PyLib: exited static initializer");
